pub struct GuestPhysMemSpace { /* private fields */ }Expand description
Guest physical memory space of a VM.
This type owns the page table that maps guest physical addresses to host physical frames. Its cursors manage 4-KiB mappings within a 48-bit guest physical address space. Each mapping must permit reads; write and execute permissions can be added independently.
This address space must be dropped outside atomic mode.
Implementations
Sourceimpl GuestPhysMemSpace
impl GuestPhysMemSpace
Sourcepub fn new() -> Result<Self>
pub fn new() -> Result<Self>
Creates a new guest physical memory space.
Errors
Returns an error if the CPU does not support second-stage address translation or the required invalidation operations.
Panics
Panics if called in atomic mode.
Sourcepub fn cursor<'a, G: AsAtomicModeGuard>(
&'a self,
guard: &'a G,
gpa: &Range<Gpaddr>,
) -> Result<Cursor<'a>>
pub fn cursor<'a, G: AsAtomicModeGuard>( &'a self, guard: &'a G, gpa: &Range<Gpaddr>, ) -> Result<Cursor<'a>>
Gets an immutable cursor over a guest physical address range.
The cursor behaves like a lock guard, exclusively owning a sub-tree of the page table, preventing others from creating a cursor in it. So be sure to drop the cursor as soon as possible.
The creation of the cursor may block if another cursor having an overlapping range is alive.
Sourcepub fn cursor_mut<'a, G: AsAtomicModeGuard>(
&'a self,
guard: &'a G,
gpa: &Range<Gpaddr>,
) -> Result<CursorMut<'a>>
pub fn cursor_mut<'a, G: AsAtomicModeGuard>( &'a self, guard: &'a G, gpa: &Range<Gpaddr>, ) -> Result<CursorMut<'a>>
Gets a mutable cursor over a guest physical address range.
The same as Self::cursor, the cursor behaves like a lock guard,
exclusively owning a sub-tree of the page table, preventing others
from creating a cursor in it. So be sure to drop the cursor as soon as
possible.
The creation of the cursor may block if another cursor having an overlapping range is alive.